## Copyright (C) 2003 Csaba Szepesvari
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

## -*- texinfo -*-
## @deftypefn {Function File} {[@var{r1},@var{r2}]=} FUNNAME (@var{ARG1}, @var{ARG2}, ..)
## INCLUDE HERE A SHORT DESCRIPTION
##
## Default values are @var{a} = 0, @var{b} = 1.
##
## Example usage:
##
## @var{RET} = FUNNAME (@var{ARG1}, @var{ARG2}, ..)
## @end deftypefn

## Author: Csaba Szepesvari (www.sztaki.hu/~szcsaba)
## Description: PDF of the uniform distribution
## Keywords: 

function [rp,prob_success] = rejection_rnd(dens_pdf,prop_pdf,prop_rnd,mul,N,show)
if (nargin<6)
  show=0;
endif

rp = zeros(N,1);
k = 0;
m = N;
g = 0;
remaining = N;
prob_success = 1;
while (remaining>0)
  x = feval(prop_rnd,m);                  if (show==1) x=sort(x); endif; %plot(x); pause;
  y = feval(prop_pdf,x) .* mul;           if (show==1) plot(x,y); hold on; endif;
  u = uniform_rnd( zeros(m,1), y, m, 1 ); if (show==1) plot(x,u,'@'); endif;
  v = feval(dens_pdf,x);                  if (show==1) plot(x,v); endif;
  r = x(find(u<v));                       if (show==1) plot(r,u(find(u<v)),'@'); printf("Look at the graph produced and press any key to proceed when ready\n"); pause; hold off; clg; endif;
  s = min(remaining,length(r));
  if (s>0)
    % append samples
    rp(k+1:k+s) = r(1:s);
    k = k + s;
    g = g + m;
    prob_success = min(1, k/g+1E-5 );
    remaining = N-k;
    m = round(remaining/prob_success+1);
  endif
endwhile

endfunction
